// Winamp win2k alpha plugin
// Copyright (C) 2001, Michael Herf.  All rights reserved.

// portions from Justin's general purpose SDK:
// Copyright (C) 1997, Justin Frankel/Nullsoft

// Note: you need the Platform SDK (or VC7) to compile this:
// http://msdn.microsoft.com/library/psdk/portals/mainport.htm

#define _WIN32_WINNT	0x0500
#define WINVER	0x0500

#include <windows.h>
#include <process.h>
#include <stdio.h>
#include <commctrl.h>
#include "resource.h"
#include "gen.h"

// index '0' is invalid, but we leave it as scratch space. :)
int timestart[5];
int lastop[5];
HWND gwin[5];
void *procs[5];

const char *names[] = {
	"NotAWinampWindow", "Winamp v1.x", "Winamp PE", "Winamp EQ", "Winamp MB"
};

const UINT uptimer = 0x12345;
const UINT dntimer = 0x12346;
int minop = 155;


BOOL WINAPI _DllMainCRTStartup(HANDLE hInst, ULONG ul_reason_for_call, LPVOID lpReserved)
{
	return TRUE;
}

BOOL CALLBACK ConfigProc(HWND hwndDlg,UINT uMsg,WPARAM wParam,LPARAM lParam);
void config();
void quit();
int init();
void config_read();
char szAppName[] = "Win2k Alpha (Herf)";

winampGeneralPurposePlugin plugin =
{
	GPPHDR_VER,
	"",
	init,
	config,
	quit,
};

void config_write();
void config_read();
void UpdateWindows();

#define SetSlider(w, x, y, z)	SendDlgItemMessage(hwndDlg,w,x,y,z)

void UpdateText(HWND wn)
{
	char ch[80];
	itoa(minop, ch, 10);
	SetWindowText(GetDlgItem(wn, IDC_OPTEXT), ch);
}

static BOOL CALLBACK dproc(HWND hwndDlg, UINT uMsg, WPARAM wParam,LPARAM lParam)
{
	switch (uMsg)
	{
		case WM_INITDIALOG:

			SetSlider(IDC_OPSLIDER, TBM_SETRANGEMIN, 0, 32);
			SetSlider(IDC_OPSLIDER, TBM_SETRANGEMAX, 0, 255);
			SetSlider(IDC_OPSLIDER, TBM_SETPOS, 1, minop);

			UpdateText(hwndDlg);

			return 0;

		case WM_HSCROLL:

			{
				HWND swnd = (HWND)lParam;
				if (swnd == GetDlgItem(hwndDlg, IDC_OPSLIDER)) {
					minop = SendDlgItemMessage(hwndDlg, IDC_OPSLIDER, TBM_GETPOS, 0, 0);

					UpdateText(hwndDlg);
					UpdateWindows();
				}
			}

		  return 0;

		case WM_CLOSE:
			DestroyWindow(hwndDlg);
	}
	return 0;
}

void config()
{
	//DialogBox(plugin.hDllInstance, MAKEINTRESOURCE(IDD_OPDLG), plugin.hwndParent, dproc);
	DialogBox(plugin.hDllInstance, MAKEINTRESOURCE(IDD_OPDLG), NULL, dproc);
}

void quit()
{
	config_write();
}

int GetWindowIndex(HWND h)
{
	int i;
	for (i = 1; i < 5; i++) {
		if (gwin[i] == h) return i;
	}
	return 0;
}

VOID CALLBACK TProc(HWND hwnd, UINT message, UINT id, DWORD time);

LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	void *proc;
	int wi = GetWindowIndex(hwnd);
	if (!wi) {
		// yikes!
		return 1;
	}

	proc = procs[wi];

	if (message == WM_ACTIVATE) {
		int act = LOWORD(wParam);
		UINT usetimer = dntimer * wi;
		if (act == WA_ACTIVE || act == WA_CLICKACTIVE) usetimer = dntimer * wi;
		else usetimer = uptimer * wi;

		timestart[wi] = GetTickCount();
		KillTimer(hwnd, uptimer * wi);
		KillTimer(hwnd, dntimer * wi);
		SetTimer(hwnd, usetimer, 10, TProc);
	}

	if (message == WM_DESTROY) {
		KillTimer(hwnd, uptimer * wi);
		KillTimer(hwnd, dntimer * wi);
		gwin[wi] = 0;
		procs[wi] = 0;
	}

	if (wi && proc != WndProc) {
		return CallWindowProc(proc, hwnd, message, wParam, lParam);
	}

	return 0;
}

void ResetWindows()
{
	int i;
	for (i = 0; i < 5; i++) {
		gwin[i] = 0;
		timestart[i] = -1;
		lastop[i] = 0;
		procs[i] = 0;
	}
}

void FindWindows()
{
	int i;
	for (i = 1; i < 5; i++) {
		if (!gwin[i]) {
			gwin[i] = FindWindow(names[i], NULL);
			if (gwin[i]) {
				SetWindowLong(gwin[i], GWL_EXSTYLE,
				        GetWindowLong(gwin[i], GWL_EXSTYLE) | WS_EX_LAYERED);
				SetLayeredWindowAttributes(gwin[i], 0, (i == 1) ? 255 : minop, LWA_ALPHA);

				procs[i] = (void*)GetWindowLong(gwin[i], GWL_WNDPROC);
				SetWindowLong(gwin[i], GWL_WNDPROC, (long)WndProc);
			}
		}
	}
}

// for dlgproc
void UpdateWindows()
{
	int i;
	for (i = 1; i < 5; i++) {
		if (gwin[i]) {
			SetLayeredWindowAttributes(gwin[i], 0, minop, LWA_ALPHA);
		}
	}
}

// Note: I'm not stupid, just avoiding floating point to make exe size smaller
//   (VC's CRT jumps in with an extra 20k otherwise)
VOID CALLBACK TProc(HWND hwnd, UINT message, UINT id, DWORD time)
{
	int wi = GetWindowIndex(hwnd);
	if (!wi) return;

	if (id == (uptimer * wi) || id == (dntimer * wi)) {
		if (message == WM_TIMER) {
			// fade
			unsigned int op;
			int fadetime = 1000;
			int fade;

			///$$$?
			FindWindows();

			if (id == dntimer) fadetime = 200;

			fade = 1000 * (time - timestart[wi]) / fadetime;

			if (fade > 1000) {
				KillTimer(hwnd, id);
				if (id == (dntimer * wi) && lastop[wi] == 255) return;
				if (id == (uptimer * wi) && lastop[wi] == minop) return;

				fade = 1000;
			}

			// smooth step
			fade = fade * fade * (3000 - 2 * fade) / 1000000;

			// map fade
			if (id == (uptimer * wi)) {
				op = 255 - (255 - minop) * fade / 1000;
			} else {	// dntimer
				op = minop + (255 - minop) * fade / 1000;
			}

			lastop[wi] = op;
			SetLayeredWindowAttributes(hwnd, 0, (unsigned char)op, LWA_ALPHA);
		}
	}
}

void config_read()
{
	char ini_file[MAX_PATH], *p;
	GetModuleFileName(plugin.hDllInstance,ini_file,sizeof(ini_file));
	p=ini_file+lstrlen(ini_file);
	while (p >= ini_file && *p != '\\') p--;
	if (++p >= ini_file) *p = 0;
	lstrcat(ini_file,"plugin.ini");

	minop = GetPrivateProfileInt(szAppName,"Opacity", minop, ini_file);
}

void config_write()
{
	char ini_file[MAX_PATH], *p;
	char string[32];
	GetModuleFileName(plugin.hDllInstance,ini_file,sizeof(ini_file));
	p=ini_file+lstrlen(ini_file);
	while (p >= ini_file && *p != '\\') p--;
	if (++p >= ini_file) *p = 0;
	lstrcat(ini_file,"plugin.ini");

	wsprintf(string,"%d",minop);
	WritePrivateProfileString(szAppName,"Opacity", string, ini_file);
}

int init()
{
	config_read();

	ResetWindows();
	FindWindows();

	{
		static char c[512];
		char filename[512],*p;
		GetModuleFileName(plugin.hDllInstance,filename,sizeof(filename));
		p = filename+lstrlen(filename);
		while (p >= filename && *p != '\\') p--;
		wsprintf((plugin.description=c),"%s (%s)",szAppName,p+1);
	}

	return 0;
}

__declspec( dllexport ) winampGeneralPurposePlugin * winampGetGeneralPurposePlugin()
{
	return &plugin;
}

